#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

"""
test_django-jsonsuit
------------

Tests for `django-jsonsuit`.
"""

from django.test import TestCase
from django.template import Context, Template

from jsonsuit.app_settings import (
    WIDGET_MEDIA_JS, WIDGET_MEDIA_CSS,
    READONLY_WIDGET_MEDIA_JS, READONLY_WIDGET_MEDIA_CSS
)
from tests.forms import TestForm, ReadonlyTestForm

import django
import re


class TestJSONSuitWidget(TestCase):

    def setUp(self):
        self.form = TestForm(data={'stats': {'rookies': 10, 'newbies': 50, 'experts': 2}})

    def test_widget_html(self):
        self.assertTrue(self.form.is_valid())
        self.assertTrue(self.form.fields.get('stats').widget.media['js'], list(WIDGET_MEDIA_JS))
        self.assertTrue(self.form.fields.get('stats').widget.media['css'], list(WIDGET_MEDIA_CSS))
        html = self.form.as_table()
        self.assertIn('<div class="jsonsuit editable" data-jsonsuit="stats">', html)
        self.assertIn('<button type="button" class="toggle button" data-raw="Raw" data-suit="Suit">Raw</button>', html)
        self.assertIn('<textarea ', html)
        self.assertIn('<div class="suit">\n    <pre><code class="language-json" data-raw="', html)

    def tearDown(self):
        pass


class TestReadonlyJSONSuitWidget(TestCase):

    def setUp(self):
        self.form = ReadonlyTestForm(data={'stats': {'rookies': 10, 'newbies': 50, 'experts': 2}})

    def test_widget_html(self):
        self.assertTrue(self.form.is_valid())
        self.assertTrue(self.form.fields.get('stats').widget.media['js'], list(READONLY_WIDGET_MEDIA_JS))
        self.assertTrue(self.form.fields.get('stats').widget.media['css'], list(READONLY_WIDGET_MEDIA_CSS))
        html = self.form.as_table()
        self.assertIn('<div class="jsonsuit readonly" data-jsonsuit="stats">', html)
        self.assertNotIn('<button type="button" class="toggle button" data-raw="Raw" data-suit="Suit">Raw</button>', html)  # noqa
        self.assertNotIn('<textarea ', html)
        self.assertIn('<div class="suit">\n    <pre><code class="language-json" data-raw="', html)

    def tearDown(self):
        pass


class TestJSONSuitTemplateTag(TestCase):
    def test_jsonsuit_tag_dict(self):
        "The jsonsuit template tag retrieves a dict to render as JSON with the name 'dict_test'."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit data 'dict_test' %}"
        ).render(Context({'data': {'stats': ['rookies', 'newbies', 'experts']}}))
        self.assertEqual(out,
"""<div class="jsonsuit readonly" data-jsonsuit="dict_test">
  <div class="suit">
    <pre><code class="language-json" data-raw="{&quot;stats&quot;: [&quot;rookies&quot;, &quot;newbies&quot;, &quot;experts&quot;]}"></code></pre>
  </div>
</div>
""")  # noqa

    def test_jsonsuit_tag_string(self):
        "The jsonsuit template tag retrieves a string to render as JSON with the name 'string_test'."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit data 'string_test' %}"
        ).render(Context({'data': '{"stats": ["rookies", "newbies", "experts"]}'}))
        self.assertEqual(out,
"""<div class="jsonsuit readonly" data-jsonsuit="string_test">
  <div class="suit">
    <pre><code class="language-json" data-raw="{&quot;stats&quot;: [&quot;rookies&quot;, &quot;newbies&quot;, &quot;experts&quot;]}"></code></pre>
  </div>
</div>
""")  # noqa

    def test_jsonsuit_tag_empty_string(self):
        "The jsonsuit template tag retrieves an empty string to render as JSON with the name 'empty_string_test'."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit data 'empty_string_test' %}"
        ).render(Context({'data': '""'}))
        self.assertEqual(out,
"""<div class="jsonsuit readonly" data-jsonsuit="empty_string_test">
  <div class="suit">
    <pre><code class="language-json" data-raw="&quot;&quot;"></code></pre>
  </div>
</div>
""")  # noqa

    def test_jsonsuit_tag_uuid(self):
        "The jsonsuit template tag retrieves a dict to render as JSON without a name and should therefore use a uuid."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit data %}"
        ).render(Context({'data': {'stats': ["rookies", "newbies", "experts"]}}))
        pattern = re.compile(r'data-jsonsuit="[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12}')  # noqa
        self.assertTrue(pattern.search(out))

    def test_jsonsuit_tag_js(self):
        "The jsonsuit template tag js default includes."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit_js %}"
        ).render(Context())
        if django.get_version().startswith('3.2'):
            self.assertEqual(out, '<script type="text/javascript" src="/static/jsonsuit/js/prism.js"></script>\n'
                                  '<script type="text/javascript" src="/static/jsonsuit/js/readonly-jsonsuit.js"></script>')  # noqa
        else:
            self.assertEqual(out, '<script src="/static/jsonsuit/js/prism.js"></script>\n'
                                  '<script src="/static/jsonsuit/js/readonly-jsonsuit.js"></script>')

    def test_jsonsuit_tag_css(self):
        "The jsonsuit template tag css default includes."
        out = Template(
            "{% load jsonsuit %}"
            "{% jsonsuit_css %}"
        ).render(Context())
        if django.get_version().startswith('3.2'):
            self.assertEqual(out, '<link href="/static/jsonsuit/css/prism-default.css" type="text/css" media="all" rel="stylesheet" />\n'  # noqa
                                  '<link href="/static/jsonsuit/css/jsonsuit.css" type="text/css" media="all" rel="stylesheet" />')  # noqa
        else:
            self.assertEqual(out, '<link href="/static/jsonsuit/css/prism-default.css" media="all" rel="stylesheet">\n'  # noqa
                                  '<link href="/static/jsonsuit/css/jsonsuit.css" media="all" rel="stylesheet">')  # noqa



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from __future__ import unicode_literals

import json
import uuid

from django.template import Library
from django.utils.safestring import mark_safe

from ..widgets import ReadonlyJSONSuit


register = Library()


@register.simple_tag
def jsonsuit_css():
    widget = ReadonlyJSONSuit()
    return widget.media['css']


@register.simple_tag
def jsonsuit_js():
    widget = ReadonlyJSONSuit()
    return widget.media['js']


@register.simple_tag
def jsonsuit(obj, name=None):
    if name is None:
        name = uuid.uuid4()
    if isinstance(obj, str) and obj:
        obj = json.loads(obj)
    widget = ReadonlyJSONSuit()
    return mark_safe(widget.render(
        name=name,
        value=json.dumps(obj, ensure_ascii=False)))



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


def check_theme(name):
    if name in AVAILABLE_THEMES:
        return name
    else:
        raise ImproperlyConfigured(
            'JSONSUIT_WIDGET_THEME must be one of {}'.format(
                ', '.join(AVAILABLE_THEMES)))


AVAILABLE_THEMES = getattr(settings, 'JSONSUIT_AVAILABLE_THEMES', ['coy',
                                                                   'dark',
                                                                   'default',
                                                                   'funky',
                                                                   'okaidia',
                                                                   'solarizedlight',
                                                                   'twilight',
                                                                   'tomorrow'])
WIDGET_THEME = getattr(settings, 'JSONSUIT_WIDGET_THEME', 'default')

SYNTAX_HIGHLIGHTER_JS = getattr(settings, 'JSONSUIT_SYNTAX_HIGHLIGHTER_JS', ('jsonsuit/js/prism.js',))
SYNTAX_HIGHLIGHTER_CSS = getattr(settings, 'JSONSUIT_SYNTAX_HIGHLIGHTER_CSS', (
    'jsonsuit/css/prism-{}.css'.format(check_theme(WIDGET_THEME)),)
)

WIDGET_MEDIA_JS = getattr(settings, 'JSONSUIT_WIDGET_MEDIA_JS', SYNTAX_HIGHLIGHTER_JS + (
    'jsonsuit/js/jsonsuit.js',)
)
WIDGET_MEDIA_CSS = getattr(settings, 'JSONSUIT_WIDGET_MEDIA_CSS', {
    'all': SYNTAX_HIGHLIGHTER_CSS + ('jsonsuit/css/jsonsuit.css',)
})

READONLY_WIDGET_MEDIA_JS = getattr(settings, 'JSONSUIT_READONLY_WIDGET_MEDIA_JS', SYNTAX_HIGHLIGHTER_JS + (
    'jsonsuit/js/readonly-jsonsuit.js',)
)
READONLY_WIDGET_MEDIA_CSS = getattr(settings, 'JSONSUIT_READONLY_WIDGET_MEDIA_CSS', {
    'all': SYNTAX_HIGHLIGHTER_CSS + ('jsonsuit/css/jsonsuit.css',)
})



# -*- coding: utf-8 -*-
from django.urls import path
from django.views.generic import TemplateView

app_name = 'jsonsuit'
urlpatterns = [
    path('', TemplateView.as_view(template_name="base.html")),
]



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import

from django import forms

from jsonsuit.widgets import JSONSuit, ReadonlyJSONSuit


class TestForm(forms.Form):
    stats = forms.CharField(widget=JSONSuit)


class ReadonlyTestForm(forms.Form):
    stats = forms.CharField(widget=ReadonlyJSONSuit)



from django.forms import widgets
from django.template.loader import render_to_string

from .app_settings import (
    WIDGET_MEDIA_JS, WIDGET_MEDIA_CSS,
    READONLY_WIDGET_MEDIA_JS, READONLY_WIDGET_MEDIA_CSS
)


class JSONSuit(widgets.Textarea):
    def render(self, name, value, attrs={}, renderer=None):
        attrs.update({'class': 'hidden'})
        textarea = super(JSONSuit, self).render(name, value, attrs)
        return render_to_string('jsonsuit/widget.html', {
            'name': name, 'value': value, 'textarea': textarea})

    class Media:
        # https://docs.djangoproject.com/en/dev/topics/forms/media/#js
        js = WIDGET_MEDIA_JS
        # https://docs.djangoproject.com/en/dev/topics/forms/media/#css
        css = WIDGET_MEDIA_CSS


class ReadonlyJSONSuit(widgets.Widget):
    def render(self, name, value, attrs=None, renderer=None):
        return render_to_string('jsonsuit/readonly_widget.html', {
            'name': name, 'value': value, 'attrs': attrs})

    class Media:
        # https://docs.djangoproject.com/en/dev/topics/forms/media/#js
        js = READONLY_WIDGET_MEDIA_JS
        # https://docs.djangoproject.com/en/dev/topics/forms/media/#css
        css = READONLY_WIDGET_MEDIA_CSS



# -*- coding: utf-8 -*-
from __future__ import unicode_literals, absolute_import

from django.urls import include, path
from django.contrib import admin

urlpatterns = [
    path('admin/', admin.site.urls),
    path('', include('jsonsuit.urls', namespace='jsonsuit')),
]
